/*
 * This software is Copyright (C) 2006-2009
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact trust@f4-i.fh-hannover.de for details.
 *
 * http://trust.inform.fh-hannover.de/
 */

#include "IMCLibrary.h"

#ifdef __WINDOWS__
#  include <windows.h>
#endif // __WINDOWS__

#ifdef WIN32
#define TNC_IMC_EXPORTS
#endif


/** @file IFIMCImpl.cpp
 * TNC IF-IMC interface implementation used by TNC client.
 * Maps between C functions (IF-IMC) and C++ methods provided by
 * IMCLibrary. A static variable imcLib must be present where
 * the C calls can be forwarded to. This is achieved by using
 * TNCFHH_IMCLIBRARY_INITIALIZE macro in a concrete MyImcLibrary.cpp.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>09.05.2006 - create file (Daniel Wuttke)</li>
 *   <li>00.02.2008 - completely redesign (ib & mbs)</li>
 *   <li>08.05.2008 - change namespace (mbs)</li>
 *   <li>10.05.2008 - add some try/catch-blocks and remove some bugs (mbs)</li>
 *   <li>10.05.2008 - add getImcInstances() code (mbs)</li>
 *   <li>08.08.2008 - fixed TNC_IMC_API Bug, added #define TNC_IMC_EXPORTS  (ib)</li>
 *   <li>16.06.2009 - redesign 0.6.0 (ib)</li>
 * </ul>
 *
 * @date 09.05.2006
 * @author Daniel Wuttke
 * @author Ingo Bente (ib)
 * @author Mike Steinmetz (mbs)
 */

#ifdef __WINDOWS__

/**
 * Main method for Windows DLL platform binding. Is called by LoadLibrary, FreeLibrary.
 *
 * @param hInstance
 * @param fdwReason
 * @param pvReserved
 * @return
 */
int WINAPI DllMain(HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved) {
    return TRUE;
}
#endif // __WINDOWS__

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_Initialize(TNC_IMCID imcID, TNC_Version minVersion, TNC_Version maxVersion, TNC_Version *pOutActualVersion)
{
	// transition to C++ world
    return tncfhh::iml::imcLib.initialize(imcID, minVersion, maxVersion, pOutActualVersion);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_ProvideBindFunction(TNC_IMCID imcID, TNC_TNCC_BindFunctionPointer bindFunction)
{
	// transition to C++
	return tncfhh::iml::imcLib.provideBindFunction(imcID, bindFunction);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_NotifyConnectionChange(TNC_IMCID imcID, TNC_ConnectionID connectionID, TNC_ConnectionState newState)
{
	// transition to C++ world
    return tncfhh::iml::imcLib.notifyConnectionChange(imcID, connectionID, newState);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_BeginHandshake(TNC_IMCID imcID, TNC_ConnectionID connectionID)
{
	// transition to C++ world
    return tncfhh::iml::imcLib.beginHandshake(imcID, connectionID);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_ReceiveMessage(TNC_IMCID imcID, TNC_ConnectionID connectionID, TNC_BufferReference messageBuffer, TNC_UInt32 messageLength, TNC_MessageType messageType)
{
	// transition to C++ world
    return tncfhh::iml::imcLib.receiveMessage(imcID, connectionID, messageBuffer, messageLength, messageType);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_BatchEnding(TNC_IMCID imcID, TNC_ConnectionID connectionID)
{
	// transition to C++ world
	return tncfhh::iml::imcLib.batchEnding(imcID, connectionID);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMC_Terminate(TNC_IMCID imcID)
{
	// transition to C++ world
    return tncfhh::iml::imcLib.terminate(imcID);
}
