/*
 * This software is Copyright (C) 2006-2009
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact trust@f4-i.fh-hannover.de for details.
 *
 * http://trust.inform.fh-hannover.de/
 */

#include "IMVLibrary.h"

#ifdef __WINDOWS__
#  include <windows.h>
#endif // __WINDOWS__


/**
 * @file
 * TNC IF-IMV interface implementation used by TNC server.
 * Maps between C functions (IF-IMV) and C++ methods provided by
 * IMVLibrary. A static variable imvLib must be present where
 * the C calls can be forwarded to. This is achieved by using
 * TNCFHH_IMVLIBRARY_INITIALIZE macro in a concrete MyImvLibrary.cpp.
 *
 * <h3>Changelog:</h3>
 * <ul>
 *   <li>09.05.2006 - create class (Daniel Wuttke)</li>
 *   <li>00.02.2008 - completely redesign (ib & mbs)</li>
 *   <li>12.02.2008 - attributeFunctions set to NULL in terminate (ib)</li>
 *   <li>08.05.2008 - change namespace (mbs)</li>
 *   <li>10.05.2008 - add some try/catch-blocks and remove some bugs (mbs)</li>
 *   <li>10.05.2008 - add getImvInstances() code (mbs)</li>
 *   <li>16.06.2009 - redesign (ib)</li>
 * </ul>
 *
 * @date 09.05.2006
 * @author Daniel Wuttke
 * @author Ingo Bente (ib)
 * @author Mike Steinmetz
 *
 */

namespace tncfhh {

namespace iml {

#ifdef WIN32
#define TNC_IMV_EXPORTS
#endif

}  // namespace iml

}  // namespace tncfhh

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_Initialize(TNC_IMVID imvID, TNC_Version minVersion, TNC_Version maxVersion, TNC_Version *pOutActualVersion)
{
    // transition to C++
    return tncfhh::iml::imvLib.initialize(imvID, minVersion, maxVersion, pOutActualVersion);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_ProvideBindFunction(TNC_IMVID imvID, TNC_TNCS_BindFunctionPointer bindFunction)
{
    // transition to C++
    return tncfhh::iml::imvLib.provideBindFunction(imvID, bindFunction);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_NotifyConnectionChange(TNC_IMVID imvID, TNC_ConnectionID connectionID, TNC_ConnectionState newState)
{
    // transition to C++
    return tncfhh::iml::imvLib.notifyConnectionChange(imvID, connectionID, newState);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_SolicitRecommendation(TNC_IMVID imvID, TNC_ConnectionID connectionID)
{
	// transition to C++
	return tncfhh::iml::imvLib.solicitRecommendation(imvID, connectionID);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_ReceiveMessage(TNC_IMVID imvID, TNC_ConnectionID connectionID, TNC_BufferReference messageBuffer, TNC_UInt32 messageLength, TNC_MessageType messageType)
{
	// transition to C++
	return tncfhh::iml::imvLib.receiveMessage(imvID, connectionID, messageBuffer, messageLength, messageType);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

TNC_Result TNC_IMV_BatchEnding(TNC_IMVID imvID, TNC_ConnectionID connectionID)
{
	// transition to C++
	return tncfhh::iml::imvLib.batchEnding(imvID, connectionID);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNC_Result TNC_IMV_Terminate(TNC_IMVID imvID)
{
	// transition to C++
	return tncfhh::iml::imvLib.terminate(imvID);
}
