/*
 * This software is Copyright (C) 2006-2009
 * Fachhochschule Hannover (University of Applied Sciences and Arts)
 * Use is subject to license conditions.
 *
 * The main licensing options available are:
 *
 * Open Source Licensing. This is the appropriate option if you want to
 * share the source code of your application with everyone you
 * distribute it to, and you also want to give them the right to share
 * who uses it. If you wish to use TNC@FHH under Open Source Licensing,
 * you must contribute all your source code to the open source
 * community in accordance with the GPL Version 2 when your application
 * is distributed. See http://www.gnu.org/copyleft/gpl.html
 *
 * Commercial Licensing. This is the appropriate option if you are
 * creating proprietary applications and you are not prepared to
 * distribute and share the source code of your application.
 * Contact trust@f4-i.fh-hannover.de for details.
 * 
 * http://trust.inform.fh-hannover.de/
 */

#include "TNCCTNCSMessageXML.h"

#include "XercesString.h"

#include <stdexcept>

namespace tncfhh
{

namespace iel
{

namespace tnc1
{

#define CLASSNAME "TNCCTNCSMessageXML"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *          TNCCTNCSMessageXML::TNCCTNCSMessageXML(TNC_MessageType,          *
 *                             xercesc::DOMElement*)                         *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCCTNCSMessageXML::TNCCTNCSMessageXML(TNC_MessageType type, xercesc::DOMElement * xml)
	:TNCCTNCSMessage(type), doc(NULL)
{
	
	xercesc::DOMImplementation * impl = xercesc::DOMImplementationRegistry::getDOMImplementation(XercesString("LS").c_str());
	if (!impl)
		throw std::runtime_error("TNCCTNCSMessageXML::constructor: Xerces error: No DOMImplementation LS found!");
	this->doc = impl->createDocument();

	if (!xml)
		throw std::runtime_error("TNCCTNCSMessageXML::TNCCTNCSMessageXML: invalid parameter: null pointer");
	
	this->xml = dynamic_cast<xercesc::DOMElement*>(this->doc->importNode(xml, true));

	if (!this->xml)
		throw std::runtime_error("TNCCTNCSMessageXML::TNCCTNCSMessageXML: fatal null pointer");
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *          TNCCTNCSMessageXML::TNCCTNCSMessageXML(TNC_MessageType,          *
 *              xercesc::DOMDocument*, xercesc::DOMElement*)                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCCTNCSMessageXML::TNCCTNCSMessageXML(TNC_MessageType type, xercesc::DOMDocument * doc, xercesc::DOMElement * xml)
	:TNCCTNCSMessage(type), doc(doc), xml(xml)
{
	if (!xml || !doc) {
		doc->release();
		throw std::runtime_error("TNCCTNCSMessageXML::TNCCTNCSMessageXML: invalid parameter: null pointer");
	}
	
	if (xml->getOwnerDocument() != doc) {
		doc->release();
		throw std::runtime_error("TNCCTNCSMessageXML::TNCCTNCSMessageXML: invalid parameter: doc is not owner of xml");
	}
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *                 TNCCTNCSMessageXML::~TNCCTNCSMessageXML()                 *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
TNCCTNCSMessageXML::~TNCCTNCSMessageXML()
{
	this->doc->release();
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 *             xercesc::DOMElement* TNCCTNCSMessageXML::getXML()             *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
xercesc::DOMElement * TNCCTNCSMessageXML::getXML()
{
	return this->xml;
}

} // namespace tnc1

} // namespace iel

} // namespace tncfhh
